program emerge;  // SFB Insect Timing 2015  /Poethke Mitesser Hovestadt

		 // evolution of emergence timing

    // Complile e.g. with Free Pascal Compiler version 2.6.4 and Delphi compatibility mode enabled (tested under Linux Mint 17.1)
    // fpc -Sd emerge.pas
    // Run with
    // ./emerge

    // input file: parameters.txt
       // parameters to be delivered (each row results in one simulation run):
       // start_season : mean time of season start
       // var_season : fluctuation of season start
       // dur_season : mean duration of season
       // lifetime : mean lifetime of individuals
       // competition : intensity of competition <0 means no comp. standard is 1

    // output file: results.txt
       // results given (one row per run)
       // lifetime : lifetime of individuals (from input)
       // sigma : environmental fluctuations of season start (from input)
       // getup : mean time of emergence (mean trait)
       // varbet : variation of mean time of emergence (trait) between individuals
       // varin  : mean individual variation of emergence (mean trait)
       // size : final size of population

      

uses
  math;

const
  NINDI = 10000;                // number of individuals in simulation;
  DECA  = 300;                  // length of year in steps
  MAXGEN = 10001;               // number of generations to simulate
  MUTRAT = 0.001;               // mutation rate
  MEANER = 1000;                // number of steps for data analysis
  T0     = 1;                   // earliest possible begin of season (day)
  NOO     =2;                   // normalizing constant for bargraph
  GRAIN   = 1;                  // steps per day
  NUMBERES = 100000;            // number of store random numbers for lifetimes
  ROUNDER  = TRUE;              // decision on emergence model (continous or in discrete steps)
  SPRING   = TRUE;              // variation only in spring = True
                                // or both (spring and autumn) = False

type
  Tbee = record
  getup     : double;   // individual getup times;
  spread    : double;   // var individual getup times;
  nectar    : double;   // amount of nectar collected;
  Tcome     : double;   // time individual arrives;
  Tgo       : double;   // time individual leaves;
  end;

var

  open       : integer;                      // mean time plant opens
  varopen:    double;                        // variation of opening time
  stayopen : integer;                        // time flower stays open
  bee       : array[1..5*NINDI] of Tbee;     // array of bees;
  nspread   : array[1..5*NINDI] of double;   // var indiv. getup times of offspr
  ngetup    : array[1..5*NINDI] of double;   // individual getup times of offspr
  popsize : integer;                         // number of individuals in Pop
  meanstart,                            // mean starting time
  meanvar,                              // mean var of starting time
  meansize,                             // mean size of population
  meansig,                              // inter individual var of getup time
  mlife,                                // stepwise mortality of individuals
  sumg,                                 // sum of start times
  sumv,                                 // sum of variances
  sumsg : double;                       // variance of start times
  hc    : double;                // half sat const of competition (individuals)
  modi  : double;                // modifier for half sat constant
  lifetimes : array[1..NUMBERES] of double; //array of random draws of lifetimes
  infile,
  outfile: textfile;
  t       : integer;              // this generation
  T1, T2   : double;              // start and end of suitable season
  numi     : integer;





//------------------------------------------------------------------------------
//- function anormal. sin. distributed random numbers     -
//------------------------------------------------------------------------------
function anormal(mean, sd : double): double;
var
u, help : double;
begin
u:= 2*random-1;
help:= (2* (pi/2 - arctan(u/sqrt(1-u*u)))/pi-1);
anormal := mean+ sd*help;
end;

//------------------------------------------------------------------------------
//- function Poisson. poisson distributed random numbers     -
//------------------------------------------------------------------------------
function Poisson(lambda:real):integer;  // calculates the number of offspring
                                        // of a single female from a Poisson
                                        // distribution using the variable
                                        // "lambda" (fertility) as mean value
var
  a,b      :real;
  i        :integer;

begin                                   // start algorithm from:

if  lambda>88 then  lambda:=88;         // Law, A., Kelton, D., Simulation
 a:=exp(- lambda);                      // Modeling And Analysis, McGraw-Hill
b:=1;                                   // Book Company, New York, 1982,
i:=-1;                                  // p. 258 - 260
repeat
  i:=i+1;
   b:=b*random;
 until (b<a)or (i>199);
poisson:=i;

end;

//------------------------------------------------------------------------------
//-  Individuals die by starvation - re-organize vector of individuals
//------------------------------------------------------------------------------
Procedure starve(var indi: integer);

  begin
  bee[indi].getup:= bee[popsize].getup;
  bee[indi].spread:= bee[popsize].spread;
  bee[indi].nectar:= bee[popsize].nectar;
  bee[indi].Tcome:= bee[popsize].Tcome;
  bee[indi].Tgo:= bee[popsize].Tgo;
  dec(popsize);
  dec(indi);
  end;

//------------------------------------------------------------------------------
//-  Individuals die (if emerged too early) or feed day by day
//------------------------------------------------------------------------------
Procedure year;
var
    i,
    h : integer;                    // day of the year
    zeit: double;                   // time step
    competitors: integer;           // actual number of competitors today
    benefit    : double;            // resources harvested today
    mmlife     : double;            // dummy parameter to limit feeding
  begin
  i:=1;
  while i <= popsize do
    begin
    if bee[i].Tcome < T1 then starve(i);   //kill those coming too early
    inc(i);
    end;


  for h:=round(GRAIN*(T1-1)) to round(GRAIN*(T2+1)) do  // feeding-loop over year
    begin
    zeit:=h/GRAIN;
    competitors:=0;
    if ((zeit>T1) and (zeit<=T2)) then                  // feeding loop season
      begin
      for i:= 1 to popsize do
      if (bee[i].Tcome<zeit) and (bee[i].Tgo>=zeit) then inc(competitors); //indiv. alive
      mmlife:=min(mlife,100-varopen);
      if modi<0 then benefit := NINDI*1.0/(mmlife*GRAIN*popsize) // no competition version
      else benefit:=(2.0/(GRAIN*mmlife))*(1-competitors/(competitors+HC));// competition

      for i:=1 to popsize do                             //now feed all bees
        begin
        if ((bee[i].Tcome< zeit) and (bee[i].Tgo>=zeit)) then
          begin
          bee[i].nectar:=bee[i].nectar+benefit;
          end;
        end;

      end;  // end of season
    end;    // end of year
  end;

//------------------------------------------------------------------------------
//-  Individuals reproduce according to amount nectar collected
//------------------------------------------------------------------------------
Procedure reproduce;
  var
  i,j,
  kids,                         // kids produced
  newpopsize : integer;         // new popsize

  begin
  newpopsize:=0;
  for i:= 1 to popsize do
    begin
    kids:=poisson((bee[i].nectar));
    for j:= newpopsize+1 to newpopsize + kids do
      begin
      ngetup[j]:=bee[i].getup;
      nspread[j]:=bee[i].spread;
      if random<MUTRAT then ngetup[j]:=ngetup[j]+5*(1-2*random);
      if random<MUTRAT then nspread[j]:=nspread[j]+5*(1-2*random);
      if ngetup[j]<0 then ngetup[j]:=0;
      if nspread[j]<0 then nspread[j]:=0;
      end;
    newpopsize:=newpopsize+kids;
    end;

  popsize:=newpopsize;              // create new pop

  for i:=1 to popsize do
    begin
    bee[i].getup:=ngetup[i];
    bee[i].spread:=nspread[i];
    bee[i].nectar:=0;
    if ROUNDER then bee[i].Tcome:=1.0*round(anormal(bee[i].getup,bee[i].spread))
    else bee[i].Tcome:=anormal(bee[i].getup,bee[i].spread);
    inc(numi);
    if numi>NUMBERES then numi:=1;
    if ROUNDER then bee[i].Tgo:=1.0*round(bee[i].Tcome+lifetimes[numi])
    else bee[i].Tgo:=bee[i].Tcome+lifetimes[numi];
    end;
  end;

//------------------------------------------------------------------------------
//-  Simulate evolution of traits
//------------------------------------------------------------------------------
procedure simulate;
var
  z, i : integer;

  begin
  meanstart:=0;
  meanvar:=0;
  meansize:=0;
  meansig:=0;

  for z:= 1 to MAXGEN do    if popsize>0 then   // simulate year if pop survived
    begin
    t:=z;

     T1:=anormal(open,varopen);    //calculate start and end of season
    if T1<T0  then T1:=T0;
    if SPRING then T2:=open+stayopen   // spring version (no var of season end)
    else T2:=open+ anormal(stayopen,varopen);
    if T2>DECA  then T2:=DECA;

    year;
    reproduce;

    sumg:=0;
    sumv:=0;
    sumsg:=0;

    for i:=1 to popsize do
      begin
      sumg:=sumg+bee[i].getup;
      sumv:=sumv+bee[i].spread;
      sumsg:= sumsg+sqr(bee[i].getup);
      end;
    if popsize>0 then
      begin
      sumg:=sumg/popsize;
      sumv:=sumv/popsize;
      sumsg:=sqrt(sumsg/popsize-sqr(sumg));
      end;
    if (t>MAXGEN-MEANER) and (popsize>0) then meanstart:=meanstart+sumg/MEANER
    else meanstart:=0;
    if (t>MAXGEN-MEANER) and (popsize>0) then meansig:=meansig+sumsg/MEANER
    else meansig:=0;
    if (t>MAXGEN-MEANER) and (popsize>0) then meanvar:=meanvar+sumv/MEANER
    else meanvar:=0;
    if (t>MAXGEN-MEANER) and (popsize>0)then meansize:=meansize+popsize/MEANER
    else meansize:=0;
    end;                                 // end zeit

  writeln (outfile, mlife:6:1,'    ', varopen:6:2,'   ', meanstart:6:2,'  ',
           meansig:6:2,' ', meanvar:6:2,' ', meansize:10:2);
  end;


//------------------------------------------------------------------------------
//-  Initialize replicate simulation
//------------------------------------------------------------------------------

procedure initialize;
var  i : integer;  // counter
  radi : double;
  begin

  for i:=1 to NUMBERES  do          // create array of random lifetimes
    begin
    repeat
    radi:=random;
    until radi>0.0001;
    lifetimes[i]:=-ln(radi)*mlife;
    end;
    numi:=0;

  for i:=1 to NINDI do             // initialize individuals
    begin
    bee[i].getup:=open-10+Random*(stayopen+20);
    bee[i].spread:=stayopen*random/4;
    if ROUNDER then bee[i].Tcome:=1.0*round(anormal(bee[i].getup,bee[i].spread))
    else bee[i].Tcome:=1.0*round(anormal(bee[i].getup,bee[i].spread));
    inc(numi);
    if numi>NUMBERES then numi:=1;
    if ROUNDER then bee[i].Tgo:=1.0*round(bee[i].Tcome+lifetimes[numi])
    else bee[i].Tgo:=bee[i].Tcome+lifetimes[numi];
    bee[i].nectar:=0;
    end;

  popsize:=NINDI;
  hc:=NINDI*mlife/stayopen;          //construction to calculate effect of comp.


end;

//------------------------------------------------------------------------------
//-  Begin of main program - Run all replicates with data from inputfile
//------------------------------------------------------------------------------

var
  outname : string;                              // output file (results)
  inname : string;                               // parameterfile

begin
  inname :='parameters.txt';
  outname:='results.txt';        // define outputfile
  assignfile(outfile,outname);
  rewrite(outfile);
  assignfile(infile,inname);
  reset(infile);
  readln(infile);
  writeln(outfile,'  lifetime sigma   getup   varbet  varin    size');

  while not eof(infile) do
       begin
       readln(infile,open,   // read mean start of season
                varopen,     // read var start of season
                stayopen,    // read mean duration of season
                mlife,       // read lifetime of this replicate
                modi);       // read competition (<0 means no comp.)

       initialize;            // initialize population
       simulate;              // simulate evolution
       end;


  closefile(outfile);          // close outputfile
  closefile(infile);           // close inputfile


end.

